/*!
 * @file NetworkManager.js
 *
 * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
 * @note Designed with Titanium for iOS and Android devices
 * @note This file contains propritary information
 * @copyright 2014 Riverside MFG LLC
 * @section DESCRIPTION
 *
 * This source file contains the connection manager window class which handles
 * connection to the system.
 */
var rxCallbackFunction = null;
var timeoutCallback = null;
var connectSocket = null;
var txData = false;
var rxBuffer='';
var conInterval;
var backgroundWorker;
var cryptoJS = require('utils/crypto');
var timeoutCtr = 0;

module.exports = {
    /*!
     * This function sets up the network manager
     *
     * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
     * @date 11/24/2014
     * @note Designed with Titanium API ver. 3.4.1
     * 
     * @param rxCallback Callback function for Rx Data
     * @param toCallback Callback function for timeout event
     */
    init: function(rxCallback,toCallback){
        rxCallbackFunction = rxCallback;
        timeoutCallback = toCallback;
    },
    /*!
     * This function starts a connection
     *
     * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
     * @date 11/24/2014
     * @note Designed with Titanium API ver. 3.4.1
     * 
     * @param ip IP to connect to
     * @param port Port to use with connection
     * @param key Encryption key to use
     */
    start: function(ip,port){
        Ti.API.log("(NetworkManager.js): Start Connection");
        if(connectSocket != null && connectSocket.state == Titanium.Network.Socket.CONNECTED){
            connectSocket.close();
        }
        clearInterval(backgroundWorker);
        clearInterval(conInterval);
        connectSocket = Ti.Network.Socket.createTCP({
            'host': ip,
            'port': port,
            'timeout': 1000,
            'mode':Titanium.Network.READ_WRITE_MODE,
            'connected': function (e) {
                clearInterval(conInterval);
                Ti.API.log('(NetworkManager.js): Server connection established!');
                Ti.Stream.pump(e.socket, readCallback, 8192, true);
            },
            'error': function(e){
                Ti.API.error(e.Message);
            }
        });
        backgroundWorker = setInterval(background_thread,1000);
        conInterval = setInterval(connection_thread,1000);
    },
    /*!
     * This function restarts the current connection if it exists
     *
     * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
     * @date 11/24/2014
     * @note Designed with Titanium API ver. 3.4.1
     * 
     */
    restart: function(){
        Ti.API.log("(NetworkManager.js): Restart Connection");
        if(connectSocket != null && connectSocket.host != ""){
            if(connectSocket.state == Titanium.Network.Socket.CONNECTED){
                connectSocket.close();
            }
            clearInterval(conInterval);
            conInterval = setInterval(connection_thread,1000);
        }    
    },
    /*!
     * This function  stops the current connection
     *
     * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
     * @date 11/24/2014
     * @note Designed with Titanium API ver. 3.4.1
     * 
     */
    stop: function(){
        Ti.API.log("(NetworkManager.js): Stop Connection");
        try{
            clearInterval(conInterval);
            clearInterval(backgroundWorker);
            if(connectSocket.state == Titanium.Network.Socket.CONNECTED){
                connectSocket.close();
            }
        }
        catch(e){
            
        }
    },
    /*!
     * This function sets the txBuffer for transmission
     *
     * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
     * @date 11/24/2014
     * @note Designed with Titanium API ver. 3.4.1
     * 
     * @param data TX data to send
     */
    tx: function(data){
        txData = data;
    }
};

/*!
 * This function polls for a connection until one is setup
 *
 * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
 * @date 11/24/2014
 * @note Designed with Titanium API ver. 3.4.1
 * 
 */
function connection_thread(){
    if(connectSocket.state == Titanium.Network.Socket.CLOSED || connectSocket.state == Titanium.Network.Socket.INITIALIZED || connectSocket.state == Titanium.Network.Socket.ERROR){
        connectSocket.connect();
    }
}


/*!
 * This function checks for connection timeouts
 *
 * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
 * @date 11/24/2014
 * @note Designed with Titanium API ver. 3.4.1
 * 
 */
function background_thread(){
    timeoutCtr++;
    if(timeoutCtr > 10){
        timeoutCtr = 0;
        timeoutCallback();
    }
}

/*!
 * This function is the read callback for the TCP socket
 * 
 * It reads data off the buffer and attemps to send data if avaiable 
 *
 * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
 * @date 11/24/2014
 * @note Designed with Titanium API ver. 3.4.1
 * 
 * @param iv IV used to encrypted data
 * @param data Data to decrypt
 * @param key Key to use for decryption
 * 
 * @return decrypted data string
 */
function readCallback(e) {
    var str;
    if (e.bytesProcessed == -1) {
        try{
            connectSocket.close();
        }
        catch(e){
            
        }
    }
    try {
        if(e.buffer) {
            str = e.buffer.toString().trim();
            if(str.indexOf(';') > -1){
                str = str.split(';');
                str[0] = str[0].trim();
                rxBuffer += str[0];
                timeoutCtr= 0;
                str = Titanium.Utils.base64decode(rxBuffer);
                rxBuffer = '';
                if(rxCallbackFunction != null){
                    rxCallbackFunction(str);
                }
            }
            else if(str != undefined){
                rxBuffer += str;
            }
        }
        else {
            Ti.API.error('*****ERROR: Could not read from WiFi no buffer*****');
        }
    }
    catch (ex) {
        Ti.API.error('*****ERROR: Could not read from WiFi '+ex+'*****');
    }
    
    try{
        if(typeof txData == 'string' && txData != '{}' && txData != '' ){
            Ti.API.log(txData);
            txData+="\n";
            connectSocket.write(Ti.createBuffer({value :  txData}));
            txData = "";
        }
    }
    catch (ex) {
        Ti.API.error('*****ERROR: Could not write to WiFi '+ex+'*****');
    }
}

/*!
 * This function decrypts data using AES CFB type encryption.  
 *
 * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
 * @date 11/24/2014
 * @note Designed with Titanium API ver. 3.4.1
 * 
 * @param iv IV used to encrypted data
 * @param data Data to decrypt
 * @param key Key to use for decryption
 * 
 * @return decrypted data string
 */
function decryptData(iv,data,key){
    iv = cryptoJS.enc.Base64.parse(iv.toString().trim().replace(/\s/g, ''));
    data = cryptoJS.enc.Base64.parse(data.toString().trim().replace(/\s/g, ''));
    if(key.length < 16){
        for(i=key.length;i<16;i++){
            key += 'X';
        }
    }
    key = Ti.Utils.base64encode(key);
    key = cryptoJS.enc.Base64.parse(key.toString().trim());
    var dec = cryptoJS.AES.decrypt({ciphertext:data,salt:''}, key, { 'iv': iv,  'padding': cryptoJS.pad.ZeroPadding ,'mode': cryptoJS.mode.CFB});
    return dec.toString(cryptoJS.enc.Utf8);
}

/*!
 * This function encrypts data using AES CFB type encryption.  
 * The encrypted data is muxex with the IV in the following format:
 * 
 * bas64(iv);base64(data)
 *
 * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
 * @date 11/24/2014
 * @note Designed with Titanium API ver. 3.4.1
 * 
 * @param data Data to encrypt
 * @param key Key to use for encryption
 * 
 * @return encrypted data string
 */
function encryptData(data,key){
    if(key.length < 16){
        for(i=key.length;i<16;i++){
            key += 'X';
        }
    }
    iv = createIv();
    ivEnc = Ti.Utils.base64encode(iv);
    ivEnc = cryptoJS.enc.Base64.parse(ivEnc.toString().trim());
    key = Ti.Utils.base64encode(key);
    key = cryptoJS.enc.Base64.parse(key.toString().trim());
    var enc = cryptoJS.AES.encrypt(data,key,{'iv':ivEnc,'padding': cryptoJS.pad.NoPadding ,'mode': cryptoJS.mode.CFB});
    return (Ti.Utils.base64encode(iv)+";"+enc.ciphertext.toString(cryptoJS.enc.Base64));
}

/*!
 * This function creates an IV for use with with encryption functions
 *
 * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
 * @date 11/24/2014
 * @note Designed with Titanium API ver. 3.4.1
 * 
 * @return 16 digit random IV
 */
function createIv(){
    var id = Ti.Platform.createUUID();
    return id.substr(0,16);
}
