/*!
 * @file ActionControllers.js
 *
 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 * @note Designed with Titanium for iOS and Android devices
 * @note This file contains propritary information
 * @copyright 2015 Muncie Power Products, Inc
 * @section DESCRIPTION
 *
 * This source file contains the connection manager window class which handles
 * connection to the system.
 */

var uiStyle = require('UI/style');
var sliderData = null;
var sliderRun = false;

/*!
 * This class creates a label with an onclick data tx action
 * that causes a progress bar to show up until a condition is met
 *
 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 * @date 11/17/2015
 * @note Designed with Titanium API ver. 4.x
 * 
 * @param style Label style object
 * @param send function to send tx data
 */
exports.CommandElement = function(style,send,label){
    var self = this,txId;
    if(label){
        this.object = Titanium.UI.createLabel(style);
    }
    else{
        this.object = Titanium.UI.createButton(style);
    }
    

    this.object.addEventListener('click',function(e){
        var checkFor = generateID();
        send(checkFor);
        waitForResponse(self,checkFor);
    });
    /*!
     * This function gets/sets the value for the control
     *
 	 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 	 * @date 11/17/2015
 	 * @note Designed with Titanium API ver. 4.x
     * 
     * @param id TX ID used to check to see if TX has happened
     * 
     * @return TX ID if param is undefined
     */
    this.value = function(id){
        if(id == undefined){
            return this.txId;
        }
        this.txId = id;
    };
    /*!
     * This function sets the display
     *
	 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
     * @date 11/17/2015
 	 * @note Designed with Titanium API ver. 4.x
     * 
     * @param val Value to set into label
     */
    this.display = function(val){
        if(label){
            this.object.text = val;
        }
        else{
            this.object.title = val;
        }
    };
    /*!
     * This function returns the current object
     *
 	 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 	 * @date 11/17/2015
 	 * @note Designed with Titanium API ver. 4.x
     * 
     * @return label object
     */
    this.getObject = function(){
        return this.object;
    };
};

/*!
 * This class creates a slider button which can either be a dual slider (0 - 100 %)
 * or single slider (100%).  Slider buttons when clicked bring up a modal window
 * that allows the user to change the value.
 * 
 * The changed value is sent back to the server on modal window close and a progress
 * bar is brought up until the tx is successful.
 *
 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 * @date 11/17/2015
 * @note Designed with Titanium API ver. 4.x
 * 
 * @param style Button style object
 * @param send function to send tx data
 * @param dual true if dual slider
 * @param dataConfig Data Config Object (min, max, inc, multi, units)
 */
exports.SliderButton = function(style,send,dual,dataConfig){
    var self = this,valueContainer;
    this.object = Titanium.UI.createButton(style);
    this.object.addEventListener('click',function(e){
        runEntryWindow(self,dual,dataConfig,send);
    });
    /*!
     * This function gets/sets the value for the control
     *
 	 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 	 * @date 11/17/2015
	 * @note Designed with Titanium API ver. 4.x
     * 
     * @param val Value to set or undefined to get
     * 
     * @return button value if val undefined
     */
    this.value = function(val){
        var t;
        if(val == undefined){
           return valueContainer;
        }
        valueContainer = val;
        if(dual){
            this.object.title = val[0]+' / '+val[1]+dataConfig.units;
        }
        else{
            t = parseFloat(val * dataConfig.multi);
            if(parseInt(t) === t){
                this.object.title = (t).toString()+dataConfig.units;
            }
            else{
                this.object.title = (t).toFixed(2)+dataConfig.units;
            }
        }
    };
    /*!
     * This function returns the current object
     *
 	 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 	 * @date 11/17/2015
	 * @note Designed with Titanium API ver. 4.x
     * 
     * @return button object
     */
    this.getObject = function(){
        return this.object;
    };
    /*!
     * This function shows the current object
     *
 	 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 	 * @date 11/17/2015
	 * @note Designed with Titanium API ver. 4.x
     */
    this.show = function(){
        this.object.visible = true;
    };
    /*!
     * This function hides the current object
     *
 	 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 	 * @date 11/17/2015
	 * @note Designed with Titanium API ver. 4.x
     */
    this.hide = function(){
        this.object.visible = false;
    };
    this.dispose = function(){
        this.object = null;
    };
};

/*!
 * This class creates a toggle button.  When clicked the toggle button cycles sequentially
 * though a list of items sending the value back to the server.  While server tx is occuring
 * a progress bar is brought up.
 *
 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 * @date 11/17/2015
 * @note Designed with Titanium API ver. 4.x
 * 
 * @param style Button style object
 * @param send function to send tx data
 * @param data Array of values.  Values are an object (val, text)
 */
exports.ToggleButton = function(style,send,data){
    var self = this;
    this.dataIndex = -1;
    this.object = Titanium.UI.createButton(style);
    this.object.addEventListener('click',function(e){
        self.dataIndex++;
        if(self.dataIndex >= data.length){
            self.dataIndex = 0;
        }
        if(send != null){
        	var checkFor = data[self.dataIndex].val;
	        send(checkFor);
	        waitForResponse(self,checkFor);
        }
        else{
        	self.value(self.dataIndex);
        }
    });
    /*!
     * This function gets/sets the value for the control
     *
 	 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 	 * @date 11/17/2015
	 * @note Designed with Titanium API ver. 4.x
     * 
     * @param val Value to set or undefined to get
     * 
     * @return button value if val undefined
     */
    this.value = function(val){
        var i;
        if(val == undefined){
            if(this.dataIndex == -1){
                return null;
            }
            return data[this.dataIndex].val;
        }
        for(i=0;i<data.length;i++){
            if(data[i].val == val){
                this.dataIndex = i;
                this.object.title = data[i].text;
            }
        }
    };
    /*!
     * This function returns the current object
     *
 	 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 	 * @date 11/17/2015
	 * @note Designed with Titanium API ver. 4.x
     * 
     * @return button object
     */
    this.getObject = function(){
        return this.object;
    };
    this.dispose = function(){
        this.object = null;
    };
};


/*!
 * This class creates a text entry element button.  When clicked the button brings up
 * a modal window where the user can use the keyboard to enter in text or numbers.
 * 
 * The changed value is sent back to the server on modal window close and a progress
 * bar is brought up until the tx is successful.
 *
 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 * @date 11/17/2015
 * @note Designed with Titanium API ver. 4.x
 * 
 * @param style Button style object
 * @param send function to send tx data
 */
exports.TextEntry = function(style,send,numeric){
    var self = this;
    this.object = Titanium.UI.createButton(style);
    this.object.addEventListener('click',function(e){
        runEntryWindow(self,2,{'numeric':numeric},send);
    });
    /*!
     * This function gets/sets the value for the control
     *
 	 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 	 * @date 11/17/2015
	 * @note Designed with Titanium API ver. 4.x
     * 
     * @param val Value to set or undefined to get
     * 
     * @return button value if val undefined
     */
    this.value = function(val){
        var temp;
        if(val == undefined){
            if(numeric){
                return parseFloat(this.object.title);
            }
            return this.object.title;
        }
        if(typeof(val) != 'string'){
            val = val.toString();
        }
        this.object.title = val;
    };
    /*!
     * This function returns the current object
     *
 	 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 	 * @date 11/17/2015
	 * @note Designed with Titanium API ver. 4.x
     * 
     * @return button object
     */
    this.getObject = function(){
        return this.object;
    };
    this.dispose = function(){
        this.object = null;
    };
};



/*!
 * This class creates a text entry element button.  When clicked the button brings up
 * a modal window where the user can use the keyboard to enter in text or numbers.
 * 
 * The changed value is sent back to the server on modal window close and a progress
 * bar is brought up until the tx is successful.
 *
 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 * @date 11/17/2015
 * @note Designed with Titanium API ver. 4.x
 * 
 * @param style Button style object
 * @param send function to send tx data
 */
exports.Calibrate = function(style,send,type){
    var self = this,objVal,id;
    this.object = Titanium.UI.createButton(style);
    this.object.addEventListener('click',function(e){
    	Ti.API.log('sending');
        runCalibrationWindow(self,type,send);
    });
    /*!
     * This function gets/sets the value for the control
     *
 	 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 	 * @date 11/17/2015
 	 * @note Designed with Titanium API ver. 4.x
     * 
     * @param val Value to set or undefined to get
     * 
     * @return button value if val undefined
     */
    this.value = function(cmd){
        var temp;
        if(cmd == undefined){
            return this.id;
        }
        this.id = cmd;
    };
    
    this.data = function(val,flow){
        if(val == undefined){
            return this.objVal;
        }
        this.objVal = val;
        if(flow != undefined){
            this.objVal[5] = flow;
        }
    };
    /*!
     * This function returns the current object
     *
 	 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 	 * @date 11/17/2015
 	 * @note Designed with Titanium API ver. 4.x
     * 
     * @return button object
     */
    this.getObject = function(){
        return this.object;
    };
    this.dispose = function(){
        this.object = null;
    };
};


/*!
 * This class creates a text entry element button.  When clicked the button brings up
 * a modal window where the user can use the keyboard to enter in text or numbers.
 * 
 * The changed value is sent back to the server on modal window close and a progress
 * bar is brought up until the tx is successful.
 *
 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 * @date 11/17/2015
 * @note Designed with Titanium API ver. 4.x
 * 
 * @param style Button style object
 * @param send function to send tx data
 */
exports.WifiEdit = function(style,send){
    var self = this,objVal,data,cmd;
    self.object = Titanium.UI.createLabel(style);
    this.object.addEventListener('click',function(e){
        runWifiWindow(self,self.data,send);
    });
    /*!
     * This function gets/sets the value for the control
     *
 	 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 	 * @date 11/17/2015
 	 * @note Designed with Titanium API ver. 4.x
     * 
     * @param val Value to set or undefined to get
     * 
     * @return button value if val undefined
     */
    this.value = function(data,cmd){
        var temp;
        if(data == undefined && cmd == undefined){
            return self.cmd;
        }
        self.cmd = cmd;
        if(data == null){
        	self.data = null;
        	self.object.text = '';
        	return;
        }
        if(typeof data == 'string'){
        	temp = {};
        	temp.ssid = data;
        	temp.key = '';
        	temp.index = 0;
        	temp.sec = 0;
        	temp.hidden = false;
        	temp.add = true;
        }
        else if(data){
        	temp = {};
        	temp.ssid = data[0];
        	temp.key = data[1];
        	temp.index = data[2];
        	temp.sec = data[3];
        	temp.hidden = data[4];
        	temp.add = false;
        }
        self.data = temp;
        self.object.text = self.data.ssid;
    };
    
    this.data = function(val,flow){
        
    };
    /*!
     * This function returns the current object
     *
 	 * @author Kevin Wijesekera <kwijesekera@munciepower.com>
 	 * @date 11/17/2015
 	 * @note Designed with Titanium API ver. 4.x
     * 
     * @return button object
     */
    this.getObject = function(){
        return self.object;
    };
    this.dispose = function(){
        self.object = null;
    };
};



/*
 * Slider Window Functions - Provides popup slider functionality
 */
var wifiObjs = {};
var wifiWindow = Titanium.UI.createWindow({
    title:'WiFi Network Configuration',
    modal: true,
    zindex: 10
});
wifiWindow.add(Titanium.UI.createView({
    backgroundColor: '#6E6E6E',
    opacity: 0.8,
}));
wifiWindow.add(Titanium.UI.createView(uiStyle._extend(uiStyle.container,{
    width: 600,
    height: 300
})));
wifiWindow.children[1].add(Titanium.UI.createLabel(uiStyle._extend(uiStyle.textSmall,{
    width: '20%',
    height: 50,
    top: '1%',
    left:'0',
    text: 'SSID'
})));
wifiWindow.children[1].add(Titanium.UI.createTextField(uiStyle._extend(uiStyle.textEntry,{
    'width': '30%', 
    'top': '1%', 
    'height': 50,
    'left':'20%'
})));
wifiWindow.children[1].add(Titanium.UI.createLabel(uiStyle._extend(uiStyle.textSmall,{
    width: '20%',
    height: 50,
    top: 60,
    left:'0',
    text: 'KEY'
})));
wifiWindow.children[1].add(Titanium.UI.createTextField(uiStyle._extend(uiStyle.textEntry,{
    'width': '30%', 
    'top': 60, 
    'height': 50,
    'left':'20%'
})));
wifiWindow.children[1].add(Titanium.UI.createLabel(uiStyle._extend(uiStyle.textSmall,{
    width: '20%',
    height: 50,
    top: 120,
    left:'0',
    text: 'INDEX'
})));
wifiObjs["ind"] = new exports.ToggleButton(uiStyle._extend(uiStyle.stdButton,{title:"1",width:'30%',top: 120,left:'20%'}),
	null,
	[{val:0,text:'1'},{val:1,text:'2'},{val:2,text:'3'},{val:3,text:'4'}]);
wifiObjs["ind"].value(0);
wifiWindow.children[1].add(wifiObjs["ind"].getObject());

wifiWindow.children[1].add(Titanium.UI.createLabel(uiStyle._extend(uiStyle.textSmall,{
    width: '20%',
    height: 50,
    top: '1%',
    left: '50%',
    text: 'Security'
})));
wifiObjs["sec"] = new exports.ToggleButton(uiStyle._extend(uiStyle.stdButton,{title:"1",width:'28%',top: '1%',left:'70%'}),
	null,
	[{val:0,text:'NONE'},{val:1,text:'WEP-KEY'},{val:2,text:'WEP-64'},{val:3,text:'WEP-128'},{val:4,text:'WPA/WPA2'},{val:5,text:'WPA-EAP'},{val:6,text:'WPA2-EAP'}]);
wifiObjs["sec"].value(0);
wifiWindow.children[1].add(wifiObjs["sec"].getObject());
wifiObjs["sec"].getObject().addEventListener('click',function(e){
	switch(wifiObjs["sec"].value()){
		default:
		case 0:
			wifiWindow.children[1].children[2].visible = false;
			wifiWindow.children[1].children[3].visible = false;
			wifiWindow.children[1].children[4].visible = false;
			wifiWindow.children[1].children[5].visible = false;
		break;
		case 1:
		case 2:
		case 3:
			wifiWindow.children[1].children[2].visible = true;
			wifiWindow.children[1].children[3].visible = true;
			wifiWindow.children[1].children[4].visible = true;
			wifiWindow.children[1].children[5].visible = true;
		break;
		case 4:
			wifiWindow.children[1].children[2].visible = true;
			wifiWindow.children[1].children[3].visible = true;
			wifiWindow.children[1].children[4].visible = false;
			wifiWindow.children[1].children[5].visible = false;
		break;
	};
});

wifiWindow.children[1].add(Titanium.UI.createLabel(uiStyle._extend(uiStyle.textSmall,{
    width: '20%',
    height: 50,
    top: 60,
    left: '50%',
    text: 'Hidden'
})));
wifiObjs["hid"] = new exports.ToggleButton(uiStyle._extend(uiStyle.stdButton,{title:"No",width:'28%',top: 60,left:'70%'}),
	null,
	[{val:0,text:'No'},{val:1,text:'Yes'}]);
wifiObjs["hid"].value(0);
wifiWindow.children[1].add(wifiObjs["hid"].getObject());

wifiWindow.children[1].add(Titanium.UI.createButton(uiStyle._extend(uiStyle.stdButton,{
    title: 'Edit',
    width: '30%',
    top: '79%',
    left: '2%'
})));
wifiWindow.children[1].children[10].addEventListener('click',function(e){
    var checkFor=generateID(),
   	data = [
    	wifiWindow.children[1].children[1].value,
    	wifiWindow.children[1].children[3].value,
    	wifiObjs["ind"].value(),
    	wifiObjs["sec"].value(),
    	(wifiObjs["hid"].value()==1)?true:false,
    	(sliderData.d.add)?'':sliderData.d.ssid,
    	checkFor
    ];

    sliderData.send("wifi.edit",data);
    waitForResponse(sliderData.object,checkFor,function(){
       wifiWindow.close();
	});
});

wifiWindow.children[1].add(Titanium.UI.createButton(uiStyle._extend(uiStyle.stdButton,{
    title: 'Remove',
    width: '30%',
    top: '79%',
    left: '35%'
})));
wifiWindow.children[1].children[11].addEventListener('click',function(e){
    var checkFor=generateID();
    
    if(sliderData.d.add){
    	return;
    }

    sliderData.send("wifi.del",[sliderData.d.ssid,checkFor]);
    waitForResponse(sliderData.object,checkFor,function(){
       wifiWindow.close();
	});
});
wifiWindow.children[1].add(Titanium.UI.createButton(uiStyle._extend(uiStyle.stdButton,{
    title: 'Cancel',
    width: '30%',
    top: '79%',
    left: '68%'
})));
wifiWindow.children[1].children[12].addEventListener('click',function(e){
    wifiWindow.close();
});

function runWifiWindow(obj,data,send){
	if(!data){
		return;
	}
	sliderData = {'d':data,'send':send,'object': obj};
	if(!data.add){
		wifiWindow.children[1].children[1].value = data.ssid;
	}
	else{
		wifiWindow.children[1].children[1].value = '';
	}
	wifiWindow.children[1].children[3].value = data.key;
	wifiObjs["ind"].value(data.index);
	wifiObjs["sec"].value(data.sec);
	wifiObjs["hid"].value(data.hidden);
	switch(data.sec){
		default:
		case 0:
			wifiWindow.children[1].children[2].visible = false;
			wifiWindow.children[1].children[3].visible = false;
			wifiWindow.children[1].children[4].visible = false;
			wifiWindow.children[1].children[5].visible = false;
		break;
		case 1:
		case 2:
		case 3:
			wifiWindow.children[1].children[2].visible = true;
			wifiWindow.children[1].children[3].visible = true;
			wifiWindow.children[1].children[4].visible = true;
			wifiWindow.children[1].children[5].visible = true;
		break;
		case 4:
			wifiWindow.children[1].children[2].visible = true;
			wifiWindow.children[1].children[3].visible = true;
			wifiWindow.children[1].children[4].visible = false;
			wifiWindow.children[1].children[5].visible = false;
		break;
	};
    wifiWindow.open();
}



/*
 * Slider Window Functions - Provides popup slider functionality
 */
var calWindow = Titanium.UI.createWindow({
    title:'Calibration',
    modal: true,
    zindex: 10
});
calWindow.add(Titanium.UI.createView({
    backgroundColor: '#6E6E6E',
    opacity: 0.8,
}));
calWindow.add(Titanium.UI.createView(uiStyle._extend(uiStyle.container,{
    width: 500,
    height: 400
})));
calWindow.children[1].add(Titanium.UI.createLabel(uiStyle._extend(uiStyle.textSmall,{
    width: '98%',
    height: 120,
    top: '1%',
    text: 'Run Spreader for desired ammount of time then hit the stop button below and enter the weight dumped.'
})));
calWindow.children[1].add(Titanium.UI.createLabel(uiStyle._extend(uiStyle.textSmall,{
    width: 200,
    left: 150,
    height: 55,
    top: 240,
    text: '100% Drive'
})));
calWindow.children[1].add(Titanium.UI.createButton(uiStyle._extend(uiStyle.stdButton,{
    title: 'Start',
    width: 140,
    top: 170,
    left: 100
})));

calWindow.children[1].add(Titanium.UI.createButton(uiStyle._extend(uiStyle.stdButton,{
    title: 'Recall',
    width: 140,
    top: 170,
    left: 260
})));
calWindow.children[1].add(Titanium.UI.createButton(uiStyle._extend(uiStyle.stdButton,{
    title: 'Down',
    width: 140,
    top: 240,
    left: 10
})));

calWindow.children[1].add(Titanium.UI.createButton(uiStyle._extend(uiStyle.stdButton,{
    title: 'Up',
    width: 140,
    top: 240,
    left: 350
})));
calWindow.children[1].add(Titanium.UI.createButton(uiStyle._extend(uiStyle.stdButton,{
    title: 'Ok',
    width: 140,
    top: 320,
    left: 100
})));

calWindow.children[1].add(Titanium.UI.createButton(uiStyle._extend(uiStyle.stdButton,{
    title: 'Cancel',
    width: 140,
    top: 320,
    left: 260
})));

calWindow.children[1].children[2].addEventListener('click',function(e){
    if(sliderData.opMode == 0 || sliderData.opMode == 2){
        var checkFor = generateID();
        sliderData.send("start",[sliderData.mode,sliderData.value[0],sliderData.opMode,checkFor]);
        waitForResponse(sliderData.object,checkFor,function(){
	        calWindow.children[1].children[2].title = "Stop";
	        sliderData.opMode = 1;
	        calWindow.children[1].children[3].enabled = false;
	        calWindow.children[1].children[4].enabled = false;
	        calWindow.children[1].children[5].enabled = false;
	        calWindow.children[1].children[6].enabled = false;
        });

    }
    else{
        var checkFor = generateID();
        sliderData.send("stop",[sliderData.mode,checkFor]);
        waitForResponse(sliderData.object,checkFor,function(){
	        calWindow.children[1].children[2].title = "Resume";
	        calWindow.children[1].children[3].title = "Save";
	        sliderData.opMode = 2;
	        calWindow.children[1].children[3].enabled = true;
	        calWindow.children[1].children[4].enabled = true;
	        calWindow.children[1].children[5].enabled = true;
	        calWindow.children[1].children[6].enabled = true;
        });

    }
});
calWindow.children[1].children[3].addEventListener('click',function(e){
    if(sliderData.opMode == 0){
        var checkFor = generateID();
        sliderData.send("recall",[sliderData.mode,checkFor]);
        waitForResponse(sliderData.object,checkFor,function(){
	        sliderData.opMode  = 2;
	        this.title = "Save";
	        calWindow.children[1].children[2].title = "Resume";
	        calWindow.children[1].children[6].enabled = true;
        });

    }
    else{
        var checkFor = generateID();
        sliderData.send("save",[sliderData.mode,checkFor]);
        waitForResponse(sliderData.object,checkFor);
    }
});
calWindow.children[1].children[4].addEventListener('touchstart',function(e){
    if(sliderData == null){
        return;
    }
    sliderRun = true;
    var e = function(){
        if(!sliderRun){
            return;
        }
        if(sliderData.opMode == 0){
            sliderData.value[0]--;
            if(sliderData.value[0] < sliderData.value[5][0]){
                sliderData.value[0] = sliderData.value[5][0];
            }
            calWindow.children[1].children[1].text = sliderData.value[0].toString()+'% Drive';
        }
        else if(sliderData.opMode == 2){
            switch(sliderData.mode){
                default:
                case 0:
                case 1:
                    sliderData.value[2]-=50;
                    if(sliderData.value[2] < 0){
                        sliderData.value[2] = 0;
                    }
                    calWindow.children[1].children[1].text = sliderData.value[2].toString()+' Lbs';
                break;
                case 2:
                    sliderData.value[2]-=1;
                    if(sliderData.value[2] < 0){
                        sliderData.value[2] = 0;
                    }
                    calWindow.children[1].children[1].text = sliderData.value[2].toString()+' Gals';
                break;
            };
            
        }
        setTimeout(e,250);
    };
    e();
});

calWindow.children[1].children[5].addEventListener('touchstart',function(e){
    if(sliderData == null){
        return;
    }
    sliderRun = true;
    var e = function(){
        if(!sliderRun){
            return;
        }
        if(sliderData.opMode == 0){
            sliderData.value[0]++;
            if(sliderData.value[0] > sliderData.value[5][1]){
                sliderData.value[0] = sliderData.value[5][1];
            }
            calWindow.children[1].children[1].text = sliderData.value[0].toString()+'% Drive';
        }
        else if(sliderData.opMode == 2){
            switch(sliderData.mode){
                default:
                case 0:
                case 1:
                    sliderData.value[2]+=50;
                    if(sliderData.value[2] > 65000){
                        sliderData.value[2] = 65000;
                    }
                    calWindow.children[1].children[1].text = sliderData.value[2].toString()+' Lbs';
                break;
                case 2:
                    sliderData.value[2]+=1;
                    if(sliderData.value[2] > 500){
                        sliderData.value[2] = 500;
                    }
                    calWindow.children[1].children[1].text = sliderData.value[2].toString()+' Gals';
                break;
            };
            
        }
        setTimeout(e,250);
    };
    e();
});
calWindow.children[1].children[4].addEventListener('touchend',endButtonHold);
calWindow.children[1].children[5].addEventListener('touchend',endButtonHold);
calWindow.children[1].children[6].addEventListener('click',function(e){
   var checkFor = generateID();
   sliderData.send("calc",[sliderData.mode,sliderData.value[2],checkFor]);
   waitForResponse(sliderData.object,checkFor,function(){
       calWindow.close();
   });
});
calWindow.children[1].children[7].addEventListener('click',function(e){
    calWindow.close();
});
    
function runCalibrationWindow(refObject,mode,send){
    sliderData = {'object': refObject, 'mode': mode,'value': refObject.data(),'send':send,opMode: 0};
    if(sliderData.value[0] < sliderData.value[5][0] || sliderData.value[0] > sliderData.value[5][1]){
        sliderData.value[0] = sliderData.value[5][0];
    }
    calWindow.open();
    switch(mode){
        default:
        case 0:
            calWindow.children[1].children[0].text = 'Run Auger for desired ammount of time then hit the stop button below and enter the weight dumped.';
        break;
        case 1:
            calWindow.children[1].children[0].text = 'Run Auger 2 for desired ammount of time then hit the stop button below and enter the weight dumped.';
        break;
        case 2:
            calWindow.children[1].children[0].text = 'Run Pre-Wet for desired ammount of time then hit the stop button below and enter the weight dumped.';
        break;
    };
    calWindow.children[1].children[1].text = sliderData.value[0].toString()+'% Drive';
    calWindow.children[1].children[2].title = "Start";
    calWindow.children[1].children[3].title = "Recall";
    calWindow.children[1].children[2].enabled = true;
    calWindow.children[1].children[3].enabled = true;
    calWindow.children[1].children[4].enabled = true;
    calWindow.children[1].children[5].enabled = true;
    calWindow.children[1].children[6].enabled = false;
    calWindow.children[1].children[7].enabled = true;
}


/*
 * Slider Window Functions - Provides popup slider functionality
 */
var entryWindow = Titanium.UI.createWindow({
    title:'Select Value',
    modal: true,
    zindex: 10
});
entryWindow.add(Titanium.UI.createView({
    backgroundColor: '#6E6E6E',
    opacity: 0.8,
}));
entryWindow.add(Titanium.UI.createView(uiStyle._extend(uiStyle.container,{
    width: 500,
    height: 230
})));
entryWindow.children[1].add(Titanium.UI.createLabel(uiStyle._extend(uiStyle.textSmall,{
    width: '50%',
    height: '60%',
    top: 0,
    text: '0000 LBS'
})));
entryWindow.children[1].add(Titanium.UI.createButton(uiStyle._extend(uiStyle.stdButton,{
    title: 'Up',
    width: 120,
    top: 10,
    left: 10
})));
entryWindow.children[1].add(Titanium.UI.createButton(uiStyle._extend(uiStyle.stdButton,{
    title: 'Down',
    width: 120,
    top: 85,
    left: 10
})));
entryWindow.children[1].add(Titanium.UI.createButton(uiStyle._extend(uiStyle.stdButton,{
    title: 'Up',
    width: 120,
    top: 10,
    left: 370
})));
entryWindow.children[1].add(Titanium.UI.createButton(uiStyle._extend(uiStyle.stdButton,{
    title: 'Down',
    width: 120,
    top: 85,
    left: 370
})));
entryWindow.children[1].add(Titanium.UI.createTextField(uiStyle._extend(uiStyle.textEntry,{
    'width': 300, 
    'top': 10, 
    'height': 50
})));
entryWindow.children[1].add(Titanium.UI.createButton(uiStyle._extend(uiStyle.stdButton,{
    title: 'Close',
    width: 150
})));

entryWindow.children[1].children[1].addEventListener('touchstart',function(e){
    //single - Down
    //dual - Up lower
    //{min:0,max:100,inc:1,multi:1,units:'%'}
    if(sliderData == null){
        return;
    }
    sliderRun = true;
    var e = function(){
        if(!sliderRun){
            return;
        }
        if(sliderData.mode == 1){
            sliderData.value[0] += sliderData.config.inc;
            if(sliderData.value[0] > sliderData.config.max){
                sliderData.value[0] = sliderData.config.max;
            }
            if(sliderData.value[0] >= sliderData.value[1]){
                sliderData.value[0] = sliderData.value[1]-sliderData.config.inc;
            }
        }
        else{
            sliderData.value -= sliderData.config.inc;
            if(sliderData.value < sliderData.config.min){
                sliderData.value = sliderData.config.min;
            }
        }
        setEntryDisplay();
        setTimeout(e,250);
    };
    e();
});

entryWindow.children[1].children[2].addEventListener('touchstart',function(e){
    //single - None
    //dual - Down lower
    if(sliderData == null){
        return;
    }
    sliderRun = true;
    var e = function(){
        if(!sliderRun){
            return;
        }
        sliderData.value[0] -= sliderData.config.inc;
        if(sliderData.value[0] < sliderData.config.min){
            sliderData.value[0] = sliderData.config.min;
        }
        setEntryDisplay();
        setTimeout(e,250);
    };
    e();
});

entryWindow.children[1].children[3].addEventListener('touchstart',function(e){
    //single - Up
    //dual - Up higher
    if(sliderData == null){
        return;
    }
    
    sliderRun = true;
    var e = function(){
        if(!sliderRun){
            return;
        }
        if(sliderData.mode == 1){
            sliderData.value[1] += sliderData.config.inc;
            if(sliderData.value[1] > sliderData.config.max){
                sliderData.value[1] = sliderData.config.max;
            }
        }
        else{
            sliderData.value += sliderData.config.inc;
            if(sliderData.value > sliderData.config.max){
                sliderData.value = sliderData.config.max;
            }
        }
        setEntryDisplay();
        setTimeout(e,250);
    };
    e();
    

});
entryWindow.children[1].children[4].addEventListener('touchstart',function(e){
    //single - None
    //dual - Down higher
    if(sliderData == null){
        return;
    }
    
    sliderRun = true;
    var e = function(){
        if(!sliderRun){
            return;
        }
        sliderData.value[1] -= sliderData.config.inc;
        if(sliderData.value[1] < sliderData.config.min){
            sliderData.value[1] = sliderData.config.min;
        }
        if(sliderData.value[0] >= sliderData.value[1]){
            sliderData.value[1] = sliderData.value[0]+sliderData.config.inc;
        }
        setEntryDisplay();
        setTimeout(e,250);
    };
    e();
});

entryWindow.children[1].children[6].addEventListener('click',function(e){
    
    if(sliderData.mode == 2){
    	sliderData.value = entryWindow.children[1].children[5].value;
    	if(sliderData.config.numeric){
    		sliderData.value = parseFloat(sliderData.value);
    	}
    }
    
    sliderData.send(sliderData.value);
    
    waitForResponse(sliderData.object,sliderData.value,function(){
        sliderData = null;
        entryWindow.close();
    });
});

entryWindow.children[1].children[1].addEventListener('touchend',endButtonHold);
entryWindow.children[1].children[2].addEventListener('touchend',endButtonHold);
entryWindow.children[1].children[3].addEventListener('touchend',endButtonHold);
entryWindow.children[1].children[4].addEventListener('touchend',endButtonHold);
function endButtonHold(){
    sliderRun = false;
}

function runEntryWindow(refObject,mode,dataConfig,send){
    entryWindow.open();
    sliderData = {'object': refObject, 'mode': mode, 'config':dataConfig,'value': refObject.value(),'send':send};

    if(sliderData.mode == 2){
        entryWindow.children[1].children[0].visible = false;
        entryWindow.children[1].children[1].visible = false;
        entryWindow.children[1].children[2].visible = false;
        entryWindow.children[1].children[3].visible = false;
        entryWindow.children[1].children[4].visible = false;
        entryWindow.children[1].children[5].visible = true;
        entryWindow.children[1].children[6].top = 80;
        if(dataConfig.numeric){
            entryWindow.children[1].children[5].keyboardType = Titanium.UI.KEYBOARD_NUMBER_PAD;
        }
        else{
            entryWindow.children[1].children[5].keyboardType = Titanium.UI.KEYBOARD_DEFAULT;
        }
    }
    else if(sliderData.mode == 1){
        entryWindow.children[1].height =  230;
        entryWindow.children[1].children[0].visible = true;
        entryWindow.children[1].children[1].title = 'Up';
        entryWindow.children[1].children[1].visible = true;
        entryWindow.children[1].children[2].title = 'Down';
        entryWindow.children[1].children[2].visible = true;
        entryWindow.children[1].children[3].title = 'Up';
        entryWindow.children[1].children[3].visible = true;
        entryWindow.children[1].children[4].title = 'Down';
        entryWindow.children[1].children[4].visible = true;
        entryWindow.children[1].children[5].visible = false;
        entryWindow.children[1].children[6].top = 150;
    }
    else{
        entryWindow.children[1].height =  160;
        entryWindow.children[1].children[0].visible = true;
        entryWindow.children[1].children[1].title = 'Down';
        entryWindow.children[1].children[1].visible = true;
        entryWindow.children[1].children[2].visible = false;
        entryWindow.children[1].children[3].title = 'Up';
        entryWindow.children[1].children[3].visible = true;
        entryWindow.children[1].children[4].visible = false;
        entryWindow.children[1].children[5].visible = false;
        entryWindow.children[1].children[6].top = 80;
    }
    setEntryDisplay();
}

function setEntryDisplay(){
    var t;
    if(sliderData.mode == 2){
        entryWindow.children[1].children[5].value = sliderData.value;
    }
    else if(sliderData.mode == 1){
        entryWindow.children[1].children[0].text = sliderData.value[0].toString()+' / '+sliderData.value[1].toString()+sliderData.config.units;
    }
    else{
        t = parseFloat(sliderData.value*sliderData.config.multi);
        if(parseInt(t) === t){
            entryWindow.children[1].children[0].text = (t).toString()+sliderData.config.units;
        }
        else{
            entryWindow.children[1].children[0].text = (t).toFixed(2)+sliderData.config.units;
        }
    }
}

/*
 * Activity Window Functions - Provides a loading window and a message display area
 */
var activityWindow = Titanium.UI.createWindow({
    title:'Loading...',
    modal: true,
    zindex: 20
});
activityWindow.add(Titanium.UI.createView({
    backgroundColor: '#6E6E6E',
    opacity: 0.8,
}));
activityWindow.add(Titanium.UI.createView(uiStyle._extend(uiStyle.container,{
    width: '380',
    height: '240'
})));
activityWindow.children[1].add(Titanium.UI.createActivityIndicator({
    color: '#FF0000',
    font: {fontSize:26, fontWeight:'bold'},
    message: ' Loading...',
    margin: 'auto',
    height:'100',
    width:'300'
}));
activityWindow.children[1].add(Titanium.UI.createLabel(uiStyle._extend(uiStyle.textSmall,{
    width: '100%',
    height: '60%',
    top: 0
})));
activityWindow.children[1].add(Titanium.UI.createButton(uiStyle._extend(uiStyle.stdButton,{
    title: 'OK',
    width: '40%',
    top: '65%'
})));
activityWindow.children[1].children[2].addEventListener('click',function(e){
    activityWindow.close();
});

function waitForResponse(object,check,success){
    var ctr = 0;
    
    activityWindow.open();
    activityWindow.children[1].children[1].text = "";
    activityWindow.children[1].children[1].visible = false;
    activityWindow.children[1].children[2].visible = false;
    activityWindow.children[1].children[0].show();
    
    var e = function(){
        var v;
        if(typeof(check) == 'object'){
            v = object.value();
            if(v[0] == check[0] && v[1] == check[1]){
                activityWindow.close();
                if(success != undefined){
                    success();
                }
                return;
            }
        }
        else{
            if(object.value() === check){
                activityWindow.close();
                if(success != undefined){
                    success();
                }
                return;    
            }
        }
        ctr++;
        if(ctr > 20){
            activityWindow.children[1].children[0].hide();
            activityWindow.children[1].children[1].visible = true;
            activityWindow.children[1].children[2].visible = true;
            activityWindow.children[1].children[1].text = "Could not send request to system!";
            return;
        }
        setTimeout(e,500);
    };
    setTimeout(e,500);
}

function generateID(){
    var id = Ti.Platform.createUUID();
    return id.substr(0,5);
}
