/*!
 * @file WindowManager.js
 *
 * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
 * @note Designed with Titanium for iOS and Android devices
 * @note This file contains propritary information
 * @copyright 2014 Riverside MFG LLC
 * @section DESCRIPTION
 *
 * This source file contains the window manager class which manages all of the available views:
 * loading, updating, switching, etc...
 */

var configurationMode = false;
var windowIndex = -1;
var lastWindowIndex = -1;
var windowOffset = 13;
var uiStyle = require('UI/style');
var rxBuffer = false;
var dynamic = require('UI/windows/config/DynamicConfig');
var actionController = require('UI/ActionControllers');    /*!< Action controller object for setting up actions */
var windows = {
    'main':[
        require('UI/windows/main/1-oper'),
        require('UI/windows/main/2-system'),
        require('UI/windows/main/4-datalog')
    ],
    'config':[
        require('UI/windows/config/1-system'),
        require('UI/windows/config/2-profile'),
        require('UI/windows/config/3-datalogging'),
        require('UI/windows/config/4-datalogging'),
        require('UI/windows/config/5-wifi'),
        require('UI/windows/config/6-global'),
        require('UI/windows/config/7-global'),
        require('UI/windows/config/8-spreader'),
        require('UI/windows/config/9-spreader'),
        require('UI/windows/config/10-spreader'),
        require('UI/windows/config/11-spreader'),
        require('UI/windows/config/12-spreader'),
        require('UI/windows/config/13-equipment'),
        require('UI/windows/config/14-antiice')
    ]
};

//config switching
windows.main[1].switchToConfig = switchToConfig;
windows.config[0].switchToNormal = switchToNormal;

module.exports = {
    /*!
     * This function sets up the WindowManager controller
     *
     * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
     * @date 11/24/2014
     * @note Designed with Titanium API ver. 3.4.1
     */
    'init': function(OnWindowClose){
        var i,tab;
        
        //init all windows
        for(i=0;i<windows.main.length;i++){
            windows.main[i].init(uiStyle,windowSwipeEvent,actionController);
        }
        for(i=0;i<windows.config.length;i++){
            windows.config[i].gotoPage0 = configPage0;
            windows.config[i].init(uiStyle,windowSwipeEvent,actionController);
            
            //add skip for spreader presets page
            if(windows.config[i].id == "cfg.sprd2"){
                var temp = i;
                windows.config[i].skipNextPage = function(skip){
                    windows.config[temp+1].skip = skip;  
                };
            }
        }
        dynamic.gotoPage0 = configPage0;
    },
    /*!
     * This function gets the status of the window manager
     *
     * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
     * @date 11/24/2014
     * @note Designed with Titanium API ver. 3.4.1
     * 
     * @return true if running, false if not
     */
    'isRunning': function(){
        return windowIndex != -1;
    },
    /*!
     * This function opens the root window of the window manager
     *
     * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
     * @date 11/24/2014
     * @note Designed with Titanium API ver. 3.4.1
     * 
     */
    'open': function(){
        Ti.API.log('(WindowManager.js): Opened Main Window');
        windowIndex = 0;
        configurationMode = false;
        try{
        	windows.main[0].update(rxBuffer);
        }
        catch(e){
        	
        }
        windows.main[0].open();
    },
    /*!
     * This function closes the currently opened window
     *
     * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
     * @date 11/24/2014
     * @note Designed with Titanium API ver. 3.4.1
     * 
     */
    'close': function(){
        if(windowIndex != -1){
            if(configurationMode && windowIndex >= windowOffset){
                dynamic.windows[(windowIndex-windowOffset)].close();
            }
            else{
                windows[((configurationMode)?'config':'main')][windowIndex].close();
            }
            
        }
        rxBuffer = false;
        windowIndex = -1;
    },
    /*!
     * This function pushes data unto the stack to update the current
     * window
     *
     * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
     * @date 11/24/2014
     * @note Designed with Titanium API ver. 3.4.1
     * 
     * @param data JSON data object
     */
    'pushData': function(data){
        rxBuffer = data;
        if(windowIndex != -1){
            if(configurationMode && windowIndex >= windowOffset){
		        try{
		        	dynamic.update(rxBuffer);
		        }
		        catch(e){
		        	
		        }
            }
            else{
		        try{
		        	windows[((configurationMode)?'config':'main')][windowIndex].update(rxBuffer);
		        }
		        catch(e){
		        	
		        }
                
            }
        }
    },
    /*!
     * This function gets tx data to send out from the current window
     *
     * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
     * @date 11/24/2014
     * @note Designed with Titanium API ver. 3.4.1
     * 
     * @return string of JSON encoded data
     */
    'getDataTx': function(){
        var tmp = {};
        if(windowIndex != -1){
            if(configurationMode && windowIndex >= windowOffset){
                tmp = dynamic.getDataTx();
            }
            else{
                tmp = windows[((configurationMode)?'config':'main')][windowIndex].getDataTx();
            }
        }
        return JSON.stringify(tmp);
    }
};

function switchToConfig(){
    Ti.API.log('(WindowManager.js): switching to config mode');
    var lastIndex = windowIndex;
    dynamic.init(uiStyle,windowSwipeEvent,rxBuffer.dy,windowOffset+1,actionController);
    dynamic.update(rxBuffer);
    windowIndex = 0;
    configurationMode = true;
    windows.config[windowIndex].update(rxBuffer);
    windows.config[windowIndex].open();
    windows.main[lastIndex].close();
}

function switchToNormal(){
    Ti.API.log('(WindowManager.js): switching to main mode');
    var lastIndex = windowIndex;
    windowIndex = 1;
    configurationMode = false;
    windows.main[windowIndex].update(rxBuffer);
    windows.main[windowIndex].open();
    windows.config[lastIndex].close();
}

function configPage0(){
    Ti.API.log('GOTO PAGE 0');
    if(windowIndex == 0){
        return;
    }
    windows.config[0].update(rxBuffer);
    windows.config[0].open();
    windows.config[windowIndex].close();
    windowIndex = 0;
}

/*!
 * This function handles the swipe event for each window to transition
 * between pages.
 * 
 * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
 * @date 11/24/2014
 * @note Designed with Titanium API ver. 3.4.1
 * 
 * @param e Event arguments
 */
function windowSwipeEvent(e){
    if(windowIndex == -1){
        return;
    }
    lastWindowIndex = windowIndex;
    swipeEventHelper(e);
}

/*!
 * This is the helper function for the window swipe event.
 * 
 * Normally the window will just increase or decrease, however,
 * if the skip flag is set in the window the helper function will be
 * called again until the skip flag on a window is cleared.
 * 
 * @author Kevin Wijesekera <kwijesekera@riversidemfg.com>
 * @date 11/24/2014
 * @note Designed with Titanium API ver. 3.4.1
 * 
 * @param e Event arguments
 */
function swipeEventHelper(e){
    if(e.direction=='right'){
        windowIndex++;
        if(windowIndex >= windows[((configurationMode)?'config':'main')].length){
            if(!configurationMode || (windowIndex - windowOffset) >= dynamic.windows.length){
                windowIndex = 0;
            }
        }
    }
    else{
        windowIndex--;
        if(windowIndex < 0){
            if(configurationMode && dynamic.windows.length != 0){
                windowIndex = windowOffset+dynamic.windows.length-1;
            }
            else{
                windowIndex = windows[((configurationMode)?'config':'main')].length-1;
            }
            
        }
    }
    Ti.API.log('Checking: '+windowIndex+', '+lastWindowIndex+', '+windows.config.length+', '+dynamic.windows.length);
    if(!configurationMode || windowIndex < windowOffset){
        if(windows[((configurationMode)?'config':'main')][windowIndex].skip){
              swipeEventHelper(e);
              return;
        }
    }

    Ti.API.log('Window Change From: '+lastWindowIndex+' To '+windowIndex);
    if(configurationMode && windowIndex >= windowOffset){
        try{
        	dynamic.update(rxBuffer);
        }
        catch(e){
        	
        }
        dynamic.windows[(windowIndex - windowOffset)].open();
        if(lastWindowIndex < windowOffset){
            windows.config[lastWindowIndex].close();
        }
        else{
            dynamic.windows[(lastWindowIndex-windowOffset)].close();
        }
    }
    else{
        if(configurationMode && windows.config[windowIndex].id == "cfg.equip"){
            windows.config[windowIndex].firstSetup(rxBuffer);
        }
        try{
        	windows[((configurationMode)?'config':'main')][windowIndex].update(rxBuffer);
        }
        catch(e){
        	
        }
        
        windows[((configurationMode)?'config':'main')][windowIndex].open();
        if(configurationMode && lastWindowIndex >= windowOffset){
            dynamic.windows[(lastWindowIndex-windowOffset)].close();
        }
        else{
            windows[((configurationMode)?'config':'main')][lastWindowIndex].close();
        }
    }
}
